/***************************************************************************
                          kcomplex.cpp  -  description
                             -------------------
    begin                : Thu Dec 9 1999
    copyright            : (C) 1999 by B. Heath Robinson
    email                : bheath@earthlink.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcomplex.h"
#include <string.h>
#include <math.h>
#include "functions.h"
#include "kscalar.h"

KComplex::KComplex(){
	sclImaginary = new KScalar(0);
	sclReal = new KScalar(0);
}
KComplex::KComplex(KComplex& newComplex){
	sclImaginary = new KScalar(*newComplex.sclImaginary);
	sclReal = new KScalar(*newComplex.sclReal);
}
KComplex::KComplex(KScalar& newScalar){
	sclImaginary = new KScalar(double(0));
	sclReal = new KScalar(newScalar);
}
KComplex::KComplex(const char * sNewValue, int iBase){
	//Verify the number has opening and closing parens
	if (sNewValue[0] == '(' && sNewValue[strlen(sNewValue)-1] == ')') {
		int iCommaPos;
		iCommaPos = (int)strchr(sNewValue,',')-(int)sNewValue;
		int i;
		char * sRealPart;
		sRealPart = new char[iCommaPos];
		char * sImaginaryPart;
		sImaginaryPart = new char[strlen(sNewValue)-iCommaPos];
		int j=0;
		for(i=1;i<iCommaPos;i++){
			sRealPart[j] = sNewValue[i];
			j++;
		}
		sRealPart[j]=NULL;
		j=0;
		for(i=iCommaPos+1;i<strlen(sNewValue)-1;i++){
			sImaginaryPart[j] = sNewValue[i];
			j++;
		}
		sImaginaryPart[j]=NULL;
		sclImaginary = new KScalar(sImaginaryPart,iBase);
		sclReal = new KScalar(sRealPart,iBase);
		delete sRealPart;
		delete sImaginaryPart;
	} else {
		sclImaginary = new KScalar(0);
		sclReal = new KScalar(0);		
	}
}
KComplex::~KComplex(){
	delete sclReal;
	delete sclImaginary;
}

double KComplex::real(){
	return sclReal->value();
}

double KComplex::imaginary(){
	return sclImaginary->value();
}

const char * KComplex::stringRepresentation(int iBase) {
  char * sRealPart;
	char * sImaginaryPart;

	sRealPart = (char*)sclReal->stringRepresentation(iBase);
	sImaginaryPart = (char*)sclImaginary->stringRepresentation(iBase);

	char * sStringRep;
	sStringRep = new char[strlen(sRealPart)+strlen(sImaginaryPart)+5];
	sStringRep[0]='(';
	sStringRep[1]=NULL;	
	sStringRep = strcat(sStringRep,sRealPart);
	sStringRep = strcat(sStringRep,",");
	sStringRep = strcat(sStringRep,sImaginaryPart);
	sStringRep = strcat(sStringRep,")");
	delete sRealPart;
	delete sImaginaryPart;

	return sStringRep;
}

void KComplex::add(KScalar& argElement){
	sclReal->setValue(real()+argElement.value());
}
void KComplex::subtract(KScalar& argElement){
	sclReal->setValue(real()-argElement.value());
}
void KComplex::multiply(KScalar& argElement){
	sclReal->setValue(real()*argElement.value());
	sclImaginary->setValue(imaginary()*argElement.value());
}
void KComplex::divide(KScalar& argElement){
	sclReal->setValue(real()/argElement.value());
	sclImaginary->setValue(imaginary()/argElement.value());
}
void KComplex::power(KScalar& argElement){
//research
}
void KComplex::root(KScalar& argElement){
//research
}


//KComplex operand
void KComplex::add(KComplex& argElement){
	sclReal->setValue(real()+argElement.real());
	sclImaginary->setValue(imaginary()+argElement.imaginary());
}
void KComplex::subtract(KComplex& argElement){
	sclReal->setValue(real()-argElement.real());
	sclImaginary->setValue(imaginary()-argElement.imaginary());
}
void KComplex::multiply(KComplex& argElement){
	double dRealPart;
	double dImaginaryPart;
	dRealPart=argElement.real()*real()-argElement.imaginary()*imaginary();
	dImaginaryPart=argElement.real()*imaginary()+real()*argElement.imaginary();
	sclReal->setValue(dRealPart);
	sclImaginary->setValue(dImaginaryPart);
}
void KComplex::divide(KComplex& argElement){
	double dRealPart;
	double dImaginaryPart;
  dRealPart=(real()*argElement.real()+imaginary()*argElement.imaginary())
										/ (argElement.real()*argElement.real()+argElement.imaginary()*argElement.imaginary());
	dImaginaryPart=(imaginary()*argElement.real()-real()*argElement.imaginary())
										/ (argElement.real()*argElement.real()+argElement.imaginary()*argElement.imaginary());
	sclReal->setValue(dRealPart);
	sclImaginary->setValue(dImaginaryPart);
}


void KComplex::power(KComplex& argElement){
//research
}
void KComplex::root(KComplex& argElement){
//research
}


//Unary Operators
void KComplex::negate(){
	sclReal->setValue(real()*-1);
	sclImaginary->setValue(imaginary()*-1);
}
void KComplex::inverse(){
	sclReal->setValue(real()
										/ (1 + imaginary()*imaginary()));
	sclImaginary->setValue(imaginary()
										/ (1 + imaginary()*imaginary()));
}
void KComplex::sine(int unit=0){
	sclReal->setValue(sin(convertAngle(real(),unit,1))*cosh(convertAngle(imaginary(),unit,1)));
	sclImaginary->setValue(cos(convertAngle(real(),unit,1))*sinh(convertAngle(imaginary(),unit,1)));
}
void KComplex::cosine(int unit=0){
//research	
}
void KComplex::tangent(int unit=0){
//research	
}
void KComplex::hypSine(){
//research	
}
void KComplex::hypCosine(){
//research	
}
void KComplex::hypTangent(){
//research	
}
void KComplex::arcsine(int unit=0){
//research	
}
void KComplex::arccosine(int unit=0){
//research	
}
void KComplex::arctangent(int unit=0){
//research	
}
void KComplex::hypArcsine(){
//research	
}
void KComplex::hypArccosine(){
//research	
}
void KComplex::hypArctangent(){
//research	
}
void KComplex::ln(){
//research	
}
void KComplex::log10(){
//research	
}
void KComplex::e2X(){
//research	
}
void KComplex::ten2X(){
//research	
}
void KComplex::compliment(){
//research	
}