/***************************************************************************
                          kpncalc.cpp  -  description
                             -------------------
    begin                : Sun Nov 21 1999
    copyright            : (C) 1999 by B. Heath Robinson
    email                : bheath@earthlink.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpncalc.h"
#include "kstack.h"
#include "kelement.h"
#include "kscalar.h"
#include "kcomplex.h"
#include <ctype.h>
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <qlistbox.h>
#include <qwidget.h>
#include <qbuttongroup.h>
#include <qkeycode.h>
#include <qclipboard.h>

KPNCalc::KPNCalc(QWidget *parent, const char *name) : QWidget(parent,name){
	
	initDialog();
  this->setCaption(i18n("K Polish Notation Calc"));
  IOBase=1;
	enableDecInput();

  angleUnit=0;
	hyp=0;
	arc=0;
	
	calcStack = new KStack;
	undoStack = new KStack;
	lblBuffer->setText("");
	
	setButtonAccels();
	setButtonPixmaps();
	connectSlots();
	
}
void KPNCalc::connectSlots(){
	connect(butOne, SIGNAL( clicked() ), this, SLOT( butOneClicked() ));
	connect(butTwo, SIGNAL( clicked() ), this, SLOT( butTwoClicked() ));
	connect(butThree, SIGNAL( clicked() ), this, SLOT( butThreeClicked() ));
	connect(butFour, SIGNAL( clicked() ), this, SLOT( butFourClicked() ));
	connect(butFive, SIGNAL( clicked() ), this, SLOT( butFiveClicked() ));
	connect(butSix, SIGNAL( clicked() ), this, SLOT( butSixClicked() ));
	connect(butSeven, SIGNAL( clicked() ), this, SLOT( butSevenClicked() ));
	connect(butEight, SIGNAL( clicked() ), this, SLOT( butEightClicked() ));
	connect(butNine, SIGNAL( clicked() ), this, SLOT( butNineClicked() ));
	connect(butZero, SIGNAL( clicked() ), this, SLOT( butZeroClicked() ));
	connect(butA, SIGNAL( clicked() ), this, SLOT( butAClicked() ));
	connect(butB, SIGNAL( clicked() ), this, SLOT( butBClicked() ));
	connect(butC, SIGNAL( clicked() ), this, SLOT( butCClicked() ));
	connect(butD, SIGNAL( clicked() ), this, SLOT( butDClicked() ));
	connect(butE, SIGNAL( clicked() ), this, SLOT( butEClicked() ));
	connect(butF, SIGNAL( clicked() ), this, SLOT( butFClicked() ));
	connect(butPeriod, SIGNAL( clicked() ), this, SLOT( butPeriodClicked() ));
	connect(butNegate, SIGNAL( clicked() ), this, SLOT( butNegateClicked() ));
	connect(butExpNotation, SIGNAL( clicked() ), this, SLOT( butExpNotationClicked() ));
	connect(butEnter, SIGNAL( clicked() ), this, SLOT( butEnterClicked() ));
	connect(butSwap, SIGNAL( clicked() ), this, SLOT( butSwapClicked() ));
	connect(butRoll, SIGNAL( clicked() ), this, SLOT( butRollClicked() ));
	connect(butFlip, SIGNAL( clicked() ), this, SLOT( butFlipClicked() ));
	connect(butFloat, SIGNAL( clicked() ), this, SLOT( butFloatClicked() ));
	connect(butSink, SIGNAL( clicked() ), this, SLOT( butSinkClicked() ));
	connect(butDelete, SIGNAL( clicked() ), this, SLOT( butDeleteClicked() ));
	connect(butClear, SIGNAL( clicked() ), this, SLOT( butClearClicked() ));	
	connect(butBack, SIGNAL( clicked() ), this, SLOT( butBackClicked() ));
	connect(butCancel, SIGNAL( clicked() ), this, SLOT( butCancelClicked() ));
	connect(butAND, SIGNAL( clicked() ), this, SLOT( butANDClicked() ));
	connect(butOR, SIGNAL( clicked() ), this, SLOT( butORClicked() ));
	connect(butCompliment, SIGNAL( clicked() ), this, SLOT( butComplimentClicked() ));
	connect(butAdd, SIGNAL( clicked() ), this, SLOT( butAddClicked() ));	
	connect(butSubtract, SIGNAL( clicked() ), this, SLOT( butSubtractClicked() ));	
	connect(butMultiply, SIGNAL( clicked() ), this, SLOT( butMultiplyClicked() ));	
	connect(butDivide, SIGNAL( clicked() ), this, SLOT( butDivideClicked() ));	
	connect(butModulous, SIGNAL( clicked() ), this, SLOT( butModulousClicked() ));	
	connect(butInverse, SIGNAL( clicked() ), this, SLOT( butInverseClicked() ));	
	connect(butPower, SIGNAL( clicked() ), this, SLOT( butPowerClicked() ));	
	connect(butRoot, SIGNAL( clicked() ), this, SLOT( butRootClicked() ));	
	connect(butUndo, SIGNAL( clicked() ), this, SLOT( butUndoClicked() ));
	connect(butArc, SIGNAL( clicked() ), this, SLOT( butArcClicked() ));	
	connect(butHyp, SIGNAL( clicked() ), this, SLOT( butHypClicked() ));
	connect(butSin, SIGNAL( clicked() ), this, SLOT( butSinClicked() ));
	connect(butCos, SIGNAL( clicked() ), this, SLOT( butCosClicked() ));
	connect(butTan, SIGNAL( clicked() ), this, SLOT( butTanClicked() ));
	connect(butLn, SIGNAL( clicked() ), this, SLOT(	butLnClicked() ));
	connect(butLog, SIGNAL( clicked() ), this, SLOT( butLogClicked() ));
	connect(butE2X, SIGNAL( clicked() ), this, SLOT( butE2XClicked() ));
	connect(butTen2X, SIGNAL( clicked() ), this, SLOT( butTen2XClicked() ));
	connect(butFactorial, SIGNAL( clicked() ), this, SLOT( butFactorialClicked() ));
	connect(butAngle, SIGNAL( clicked() ), this, SLOT( butAngleClicked() ));
	connect(butBase, SIGNAL( clicked() ), this, SLOT( butBaseClicked() ));
	connect(butCopy, SIGNAL( clicked() ), this, SLOT( butCopyClicked() ));
	connect(butPaste, SIGNAL( clicked() ), this, SLOT( butPasteClicked() ));
	connect(butHelp, SIGNAL( clicked() ), this, SLOT( butHelpClicked() ));
	connect(butParentheses, SIGNAL( clicked() ), this, SLOT( butParenthesesClicked() ));
	connect(butComma, SIGNAL( clicked() ), this, SLOT( butCommaClicked() ));
	connect(butComplex, SIGNAL( clicked() ), this, SLOT( butComplexClicked() ));	
}

void KPNCalc::setButtonPixmaps(){
  QString qsPixmapLocation;
	qsPixmapLocation = kapp->kde_datadir() + "/kpncalc/buttons/";
	butEnter->setPixmap(QPixmap(qsPixmapLocation + "enter.xpm"));
	butFlip->setPixmap(QPixmap(qsPixmapLocation + "flip.xpm"));
	butSwap->setPixmap(QPixmap(qsPixmapLocation + "swap.xpm"));
	butRoll->setPixmap(QPixmap(qsPixmapLocation + "roll.xpm"));

	qsPixmapLocation = kapp->kde_toolbardir() + "/";
	butCancel->setPixmap(QPixmap(qsPixmapLocation + "start.xpm"));
	butBack->setPixmap(QPixmap(qsPixmapLocation + "back.xpm"));
	butClear->setPixmap(QPixmap(qsPixmapLocation + "filenew2.xpm"));
	butFloat->setPixmap(QPixmap(qsPixmapLocation + "up.xpm"));
	butSink->setPixmap(QPixmap(qsPixmapLocation + "down.xpm"));
	butDelete->setPixmap(QPixmap(qsPixmapLocation + "delete.xpm"));
	butUndo->setPixmap(QPixmap(qsPixmapLocation + "undo.xpm"));
	butCopy->setPixmap(QPixmap(qsPixmapLocation + "editcopy.xpm"));
	butPaste->setPixmap(QPixmap(qsPixmapLocation + "editpaste.xpm"));
	butHelp->setPixmap(QPixmap(qsPixmapLocation + "help.xpm"));
}
/**  */
void KPNCalc::setButtonAccels(){
	butOne->setAccel(Key_1);
	butTwo->setAccel(Key_2);
	butThree->setAccel(Key_3);
	butFour->setAccel(Key_4);
	butFive->setAccel(Key_5);
	butSix->setAccel(Key_6);
	butSeven->setAccel(Key_7);
	butEight->setAccel(Key_8);
	butNine->setAccel(Key_9);
	butZero->setAccel(Key_0);
	butA->setAccel(Key_A);
	butB->setAccel(Key_B);
	butC->setAccel(Key_C);
	butD->setAccel(Key_D);
	butE->setAccel(Key_E);
	butF->setAccel(Key_F);
	butExpNotation->setAccel(Key_X);
	butInverse->setAccel(Key_I);
	butRoot->setAccel(Key_R);
	butPower->setAccel(Key_P);
	butLn->setAccel(Key_L);
	butCompliment->setAccel(Key_AsciiCircum);
	butFactorial->setAccel(Key_Exclam);
	butNegate->setAccel(Key_N);
	butSin->setAccel(Key_S);
	butCos->setAccel(Key_O);
	butTan->setAccel(Key_T);
	butAND->setAccel(Key_Ampersand);
	butOR->setAccel(Key_Bar);
	butCompliment->setAccel(Key_M);
	butEnter->setAccel(Key_Enter);
	butPeriod->setAccel(Key_Period);
	butComma->setAccel(Key_Comma);
	butAdd->setAccel(Key_Plus);
	butSubtract->setAccel(Key_Minus);
	butMultiply->setAccel(Key_Asterisk);
	butDivide->setAccel(Key_Slash);	
	butModulous->setAccel(Key_Percent);
	butDelete->setAccel(Key_Delete);
	butClear->setAccel(CTRL+Key_L);	
	butSwap->setAccel(CTRL+Key_S);
	butFlip->setAccel(CTRL+Key_F);
	butRoll->setAccel(CTRL+Key_R);
	butFloat->setAccel(CTRL+Key_O);
	butSink->setAccel(CTRL+Key_I);
	butUndo->setAccel(CTRL+Key_Z);
	butBack->setAccel(Key_Backspace);
	butCancel->setAccel(Key_Escape);
	butAngle->setAccel(CTRL+Key_A);
	butBase->setAccel(CTRL+Key_B);
	butCopy->setAccel(CTRL+Key_C);
	butPaste->setAccel(CTRL+Key_V);
	butHelp->setAccel(Key_F1);

}
void KPNCalc::keyPressEvent(QKeyEvent * k){
	switch (k->key()) {
		case Key_Bar : butORClicked(); break;
		case Key_AsciiCircum : butComplimentClicked(); break;
		case Key_ParenLeft : butParenthesesClicked(); break;
		case Key_ParenRight : butParenthesesClicked(); break;
		case Key_Ampersand : butANDClicked(); break;
		case Key_Percent : butModulousClicked(); break;
		case Key_Exclam : butFactorialClicked(); break;
		case Key_Minus : butSubtractClicked(); break;
		case Key_Plus : butAddClicked(); break;
		case Key_Slash : butDivideClicked(); break;
		case Key_Asterisk : butMultiplyClicked(); break;
		case Key_Return : butEnterClicked(); break;
	}
}

void KPNCalc::ReloadGrid(){
	const char * charBuffer;
	char cPrevElementType=0;
	char cCurrentElementType=0;
	KElement * currentElement;
	lstStack->clear();

	currentElement = calcStack->base();
	if (currentElement) {
		cCurrentElementType=currentElement->type();
		if (currentElement->previous())
			cPrevElementType=currentElement->previous()->type();
	}
	while(currentElement != NULL){
		charBuffer = new char[11];
		charBuffer = currentElement->stringRepresentation(IOBase);
		lstStack->insertItem(charBuffer,0);	
		currentElement = currentElement->previous();
		delete charBuffer;
	}	

	setOperationsMode(cCurrentElementType,cPrevElementType);
	lstStack->setBottomItem(lstStack->count());
}

void KPNCalc::setOperationsMode(char cCurrentType, char cPrevType) {
	if (cCurrentType=='C' && cPrevType=='C') {
    setTrigOperationsEnabled(FALSE);
		setMiscUnOperationsEnabled(FALSE);
		setMiscBinOperationsEnabled(FALSE);
		setLogarithmOperationsEnabled(FALSE);
		setPowerOperationsEnabled(FALSE);
	} else if (cCurrentType=='S' && cPrevType=='S' || cCurrentType=='S' && cPrevType==0) {
    setTrigOperationsEnabled(TRUE);
		setMiscUnOperationsEnabled(TRUE);
		setMiscBinOperationsEnabled(TRUE);
		setLogarithmOperationsEnabled(TRUE);
		setPowerOperationsEnabled(TRUE);
	} else if (cCurrentType=='S' && cPrevType=='C') {
    setTrigOperationsEnabled(TRUE);
		setMiscUnOperationsEnabled(TRUE);
		setMiscBinOperationsEnabled(FALSE);
		setLogarithmOperationsEnabled(TRUE);
		setPowerOperationsEnabled(TRUE);
	} else if (cCurrentType=='C' && cPrevType=='S' || cCurrentType=='C' && cPrevType==0) {
    setTrigOperationsEnabled(FALSE);
		setMiscUnOperationsEnabled(FALSE);
		setMiscBinOperationsEnabled(FALSE);
		setLogarithmOperationsEnabled(FALSE);
		setPowerOperationsEnabled(FALSE);
	}
}

void KPNCalc::enableBinInput(){
	butTwo->setEnabled(FALSE);
	butThree->setEnabled(FALSE);
	butFour->setEnabled(FALSE);
	butFive->setEnabled(FALSE);
	butSix->setEnabled(FALSE);
	butSeven->setEnabled(FALSE);
	butEight->setEnabled(FALSE);
	butNine->setEnabled(FALSE);
	butA->setEnabled(FALSE);
	butB->setEnabled(FALSE);
	butC->setEnabled(FALSE);
	butD->setEnabled(FALSE);
	butE->setEnabled(FALSE);
	butF->setEnabled(FALSE);
	butPeriod->setEnabled(FALSE);
}
void KPNCalc::enableDecInput(){
	butTwo->setEnabled(TRUE);
	butThree->setEnabled(TRUE);
	butFour->setEnabled(TRUE);
	butFive->setEnabled(TRUE);
	butSix->setEnabled(TRUE);
	butSeven->setEnabled(TRUE);
	butEight->setEnabled(TRUE);
	butNine->setEnabled(TRUE);
	butA->setEnabled(FALSE);
	butB->setEnabled(FALSE);
	butC->setEnabled(FALSE);
	butD->setEnabled(FALSE);
	butE->setEnabled(FALSE);
	butF->setEnabled(FALSE);
	butPeriod->setEnabled(TRUE);
}
void KPNCalc::enableHexInput(){
	butTwo->setEnabled(TRUE);
	butThree->setEnabled(TRUE);
	butFour->setEnabled(TRUE);
	butFive->setEnabled(TRUE);
	butSix->setEnabled(TRUE);
	butSeven->setEnabled(TRUE);
	butEight->setEnabled(TRUE);
	butNine->setEnabled(TRUE);
	butA->setEnabled(TRUE);
	butB->setEnabled(TRUE);
	butC->setEnabled(TRUE);
	butD->setEnabled(TRUE);
	butE->setEnabled(TRUE);
	butF->setEnabled(TRUE);
	butPeriod->setEnabled(FALSE);
}

void KPNCalc::setTrigOperationsEnabled(int iState){
	butSin->setEnabled(iState);
	butCos->setEnabled(iState);
	butTan->setEnabled(iState);
	butArc->setEnabled(iState);
	butHyp->setEnabled(iState);
}

void KPNCalc::setLogarithmOperationsEnabled(int iState){
	butLn->setEnabled(iState);
	butLog->setEnabled(iState);
	butE2X->setEnabled(iState);
	butTen2X->setEnabled(iState);
}

void KPNCalc::setPowerOperationsEnabled(int iState){
	butRoot->setEnabled(iState);
	butPower->setEnabled(iState);
}

void KPNCalc::setMiscUnOperationsEnabled(int iState){
	butFactorial->setEnabled(iState);
	butCompliment->setEnabled(iState);
	butInverse->setEnabled(iState);
}

void KPNCalc::setMiscBinOperationsEnabled(int iState){
	butModulous->setEnabled(iState);
	butAND->setEnabled(iState);
	butOR->setEnabled(iState);
	butComplex->setEnabled(iState);
}

KPNCalc::~KPNCalc(){         	
	delete(calcStack);
	delete(undoStack);
}

void KPNCalc::butOneClicked(){
	RefreshStatus();
	addCharToBuffer("1");
}
void KPNCalc::butTwoClicked(){
	RefreshStatus();
	addCharToBuffer("2");
}
void KPNCalc::butThreeClicked(){
	RefreshStatus();
	addCharToBuffer("3");
}
void KPNCalc::butFourClicked(){
	RefreshStatus();
	addCharToBuffer("4");
}
void KPNCalc::butFiveClicked(){
	RefreshStatus();
	addCharToBuffer("5");
}
void KPNCalc::butSixClicked(){
	RefreshStatus();
	addCharToBuffer("6");
}
void KPNCalc::butSevenClicked(){
	RefreshStatus();
	addCharToBuffer("7");
}
void KPNCalc::butEightClicked(){
	RefreshStatus();
	addCharToBuffer("8");
}
void KPNCalc::butNineClicked(){
	RefreshStatus();
	addCharToBuffer("9");
}
void KPNCalc::butZeroClicked(){
	RefreshStatus();
	addCharToBuffer("0");
}
void KPNCalc::butAClicked(){
	RefreshStatus();
	addCharToBuffer("A");
}
void KPNCalc::butBClicked(){
	RefreshStatus();
	addCharToBuffer("B");
}
void KPNCalc::butCClicked(){
	RefreshStatus();
	addCharToBuffer("C");
}
void KPNCalc::butDClicked(){
	RefreshStatus();
	addCharToBuffer("D");
}
void KPNCalc::butEClicked(){
	RefreshStatus();
	addCharToBuffer("E");
}
void KPNCalc::butFClicked(){
	RefreshStatus();
	addCharToBuffer("F");
}
void KPNCalc::butPeriodClicked(){
	RefreshStatus();
	QString qsLblBuffer(lblBuffer->text());
	if (lblBuffer->text() == NULL) {
		addCharToBuffer(i18n("0"));
		addCharToBuffer(".");
	} else {
		if (qsLblBuffer.findRev('.') <= qsLblBuffer.findRev(','))
			addCharToBuffer(".");
		else
			lblStatus->setText(i18n("The buffer already contains a period."));
	}
}
void KPNCalc::butComplexClicked(){
	QString qsNewComplex;
	RefreshStatus();
	if (calcStack->base()!= NULL) {
		if (calcStack->base()->type()=='S') {
			if (calcStack->base()->previous()->type()=='S') {
				saveUndo();
				qsNewComplex = "(";
				qsNewComplex += calcStack->base()->stringRepresentation(IOBase);
				qsNewComplex += ",";
				qsNewComplex += calcStack->base()->previous()->stringRepresentation(IOBase);
				qsNewComplex += ")";
				calcStack->pop();
				calcStack->pop();
		    KComplex * newComplex;
				newComplex = new KComplex(qsNewComplex, IOBase);
				KElement * newElement;
				newElement = new KElement(*newComplex);
				calcStack->push(newElement);    		
				ReloadGrid();
				//delete newComplex;
				//delete newElement;
	    	//lblBuffer->setText(qsNewComplex);
			} else {
			lblStatus->setText(i18n("y must be a scalar"));
			}
		} else {
			lblStatus->setText(i18n("x must be a scalar"));
		}
	} else {
		lblStatus->setText(i18n("x and y must be scalars"));
	}
}

void KPNCalc::butCommaClicked(){
	RefreshStatus();
	if (strlen(lblBuffer->text()) && strchr(lblBuffer->text(),'(') && !strchr(lblBuffer->text(),',') && !strchr(lblBuffer->text(),')')) {
		addCharToBuffer(",");
	} else {
		lblStatus->setText(i18n("A comma must be used with parentheses for a complex number"));			
	}
}

void KPNCalc::butParenthesesClicked(){
	RefreshStatus();
	if (strlen(lblBuffer->text())==0) {
		addCharToBuffer("(");
	} else if (!isdigit(lblBuffer->text()[0]) && strchr(lblBuffer->text(),')')==NULL && strchr(lblBuffer->text(),',')!=NULL) {
			addCharToBuffer(")");
	} else {
		lblStatus->setText(i18n("Invalid complex number input."));	
	}
}

void KPNCalc::butNegateClicked(){
	QString qsLblText;
	QString qsLblBuffer;
	RefreshStatus();
	saveUndo();
	qsLblBuffer = lblBuffer->text();
	if (qsLblBuffer.length() > 0) {
		int isComplex;
		int iCommaLoc=0;
		int iELoc;
		isComplex = qsLblBuffer.contains('(');
		if (isComplex) iCommaLoc = (qsLblBuffer.find(',')==-1)?0:qsLblBuffer.find(',');
		iELoc = (qsLblBuffer.findRev('e')==-1)?0:qsLblBuffer.findRev('e');
		if (iELoc > iCommaLoc) {
			if (qsLblBuffer.find('-',iELoc)==-1) { //There is not already a '-' sign for the exponent
				qsLblText = qsLblBuffer;
				qsLblText.insert(iELoc+1,'-');				
			} else {
				qsLblText=qsLblBuffer.left(iELoc+1);
				qsLblText+=qsLblBuffer.right(qsLblBuffer.length()-iELoc-2);
			}					
			lblBuffer->setText(qsLblText);
		} else {
			int iNegPos;
			if (isComplex) {
				if (iCommaLoc) {
					iNegPos = iCommaLoc+1;
				} else {
					iNegPos=1;
				}
			} else {
				iNegPos=0;
			}
			if (qsLblBuffer.findRev('-') <= qsLblBuffer.findRev(',') ){
				qsLblText = qsLblBuffer;
				qsLblText.insert(iNegPos,'-');
			} else {
				qsLblText=qsLblBuffer.left(iNegPos);
				qsLblText+=qsLblBuffer.right(qsLblBuffer.length()-iNegPos-1);
			}
			lblBuffer->setText(qsLblText);
		}
	} else {
		calcStack->negate();
		ReloadGrid();
	}
}
void KPNCalc::butExpNotationClicked(){
	int iCommaLoc;
	QString qsLblBuffer(lblBuffer->text());
	RefreshStatus();
	iCommaLoc = qsLblBuffer.findRev(',');//==-1?-10:qsLblBuffer.findRev(',');
	if ((qsLblBuffer.length()>0 || iCommaLoc < int(qsLblBuffer.length())-1)
			&& qsLblBuffer != "(" && (iCommaLoc < int(qsLblBuffer.length())-1))
		if (qsLblBuffer.findRev(',') >= qsLblBuffer.findRev('e'))
			addCharToBuffer(i18n("e"));
		else
			lblStatus->setText(i18n("You are already in exponential notation mode."));
	else
		lblStatus->setText(i18n("You must first have a base to raise."));
}

void KPNCalc::butPasteClicked(){
	QClipboard *cb = QApplication::clipboard();
	QString qsCBText(cb->text());
	bool bOK;
	qsCBText.toDouble(&bOK);
	if (bOK)
		lblBuffer->setText(cb->text());
	else
		lblStatus->setText(i18n("You may only paste a number"));
}

void KPNCalc::butCopyClicked(){
	QClipboard *cb = QApplication::clipboard();
	cb->setText(lstStack->text(lstStack->currentItem()));
}

void KPNCalc::butEnterClicked(){
	KElement * newElement;
	RefreshStatus();
	saveUndo();
	if (bufferVerified()) {
		newElement = createNewElement(lblBuffer->text());
		calcStack->push(newElement);
		ReloadGrid();
		lblBuffer->setText(NULL);
	}
}
void KPNCalc::butSwapClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->swap();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to swap."));
	}	
}
void KPNCalc::butRollClicked(){
	int selectedItem;
	RefreshStatus();
	selectedItem=lstStack->currentItem();
	if (validateOperation(2) && selectedItem != -1) {
		saveUndo();
		calcStack->roll(lstStack->count()-selectedItem-1);
		ReloadGrid();
		lstStack->setCurrentItem(selectedItem);
	} else {
		lblStatus->setText(i18n("You need at least two arguements to roll."));
	}	

}
void KPNCalc::butFlipClicked(){
	int selectedItem;
	RefreshStatus();
	selectedItem=lstStack->currentItem();
	if (validateOperation(2) && selectedItem != -1) {
		saveUndo();
		calcStack->flip(lstStack->count()-selectedItem-1);
		ReloadGrid();
		lstStack->setCurrentItem(selectedItem);
	} else {
		lblStatus->setText(i18n("You need at least two arguements to flip."));
	}	

}
void KPNCalc::butFloatClicked(){
	int selectedItem;
	RefreshStatus();
	selectedItem=lstStack->currentItem();
	cout << selectedItem << endl;
	if (validateOperation(2) && selectedItem != -1 && selectedItem != 0) {
		saveUndo();
		calcStack->floate(lstStack->count()-selectedItem-1);
		ReloadGrid();
		lstStack->setCurrentItem(selectedItem-1);
	} else {
		if (validateOperation(2))
			if (selectedItem != -1)
				lblStatus->setText(i18n("You cannot float the top element."));
			else
				lblStatus->setText(i18n("Please select an element to float."));
		else
			lblStatus->setText(i18n("You need two arguements to float."));

	}	

}
void KPNCalc::butSinkClicked(){
	int selectedItem;
	RefreshStatus();
	selectedItem=lstStack->currentItem();
	cout << selectedItem << endl;
	if (validateOperation(2) && selectedItem != -1 && unsigned(selectedItem) != lstStack->count()-1) {
 		saveUndo();
		calcStack->sink(lstStack->count()-selectedItem-1);
		ReloadGrid();
		lstStack->setCurrentItem(selectedItem+1);
	} else {
		if (validateOperation(2))
			if (selectedItem != -1)
				lblStatus->setText(i18n("You cannot sink the bottom element."));
			else
				lblStatus->setText(i18n("Please select an element to sink."));
		else
			lblStatus->setText(i18n("You need two arguements to sink."));
	}	

}
void KPNCalc::butDeleteClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->pop();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need an arguement to delete."));
	}	
}
void KPNCalc::butClearClicked(){
	RefreshStatus();
	saveUndo();
	calcStack->clear();
	ReloadGrid();
}
void KPNCalc::butCancelClicked(){
	lblBuffer->setText(NULL);
}
void KPNCalc::butBackClicked(){
	QString qsLblText(lblBuffer->text());
	if (qsLblText.length()>0) {
		cerr << qsLblText.findRev(',') << " " << qsLblText.length() << endl;
		if(qsLblText.findRev(',')==int(qsLblText.length())-1) butComma->setEnabled(TRUE);
		qsLblText = qsLblText.left(qsLblText.length()-1);
		lblBuffer->setText(qsLblText);
	}
}
void KPNCalc::butANDClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->AND();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to AND."));
	}	
}
void KPNCalc::butORClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->OR();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to OR."));
	}	
}
void KPNCalc::butComplimentClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->compliment();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to find compliment."));
	}	
}
void KPNCalc::butAddClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->add();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to add."));
	}	
}
void KPNCalc::butSubtractClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->subtract();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to subtract."));
	}
}
void KPNCalc::butMultiplyClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->multiply();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to multiply."));
	}
}
void KPNCalc::butDivideClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->divide();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to divide."));
	}
}
void KPNCalc::butModulousClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->modulous();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to modulous."));
	}
}
void KPNCalc::butInverseClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->inverse();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need an arguement to invert."));
	}
}
void KPNCalc::butE2XClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->e2X();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need an arguement to which to raise e."));
	}
}
void KPNCalc::butTen2XClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->ten2X();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need an arguements to to which to raise 10."));
	}
}
void KPNCalc::butRootClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->root();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to find yth root."));
	}
}
void KPNCalc::butPowerClicked(){
	RefreshStatus();
	if (validateOperation(2)) {
		saveUndo();
		calcStack->power();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to raise to yth power."));
	}
}
void KPNCalc::butArcClicked(){
	if (arc == 1)
		arc = 0;
	else
		arc = 1;
}
void KPNCalc::butHypClicked(){
	if (hyp == 1)
		hyp = 0;
	else
		hyp = 1;
}
void KPNCalc::butSinClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		if (hyp)
			if (arc) {
				calcStack->hypArcsine();
				butArc->toggle();
				butArcClicked();
			} else
				calcStack->hypSine();
		else
			if (arc) {
				calcStack->arcsine(angleUnit);
				butArc->toggle();
				butArcClicked();
			} else
				calcStack->sine(angleUnit);
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need one argument to find the sine."));
	}
}
void KPNCalc::butCosClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		if (hyp)
			if (arc) {
				calcStack->hypArccosine();
				butArc->toggle();
				butArcClicked();
			} else
				calcStack->hypCosine();
		else
			if (arc) {
				calcStack->arccosine(angleUnit);
				butArc->toggle();
				butArcClicked();
			}	else
				calcStack->cosine(angleUnit);
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need one arguements find the cosine."));
	}
}
void KPNCalc::butTanClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		if (hyp)
			if (arc) {
				calcStack->arctangent();
				butArc->toggle();
				butArcClicked();
			}	else
				calcStack->hypTangent();
		else
			if (arc) {
				calcStack->arctangent(angleUnit);
				butArc->toggle();
				butArcClicked();
			}	else
				calcStack->tangent(angleUnit);
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need one arguements find the tangent."));
	}
}
void KPNCalc::butLogClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->log10();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to raise to yth power."));
	}
}
void KPNCalc::butLnClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->ln();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to raise to yth power."));
	}
}
void KPNCalc::butFactorialClicked(){
	RefreshStatus();
	if (validateOperation(1)) {
		saveUndo();
		calcStack->factorial();
		ReloadGrid();
	} else {
		lblStatus->setText(i18n("You need two arguements to raise to yth power."));
	}
}

void KPNCalc::butUndoClicked(){
	restoreUndoStack();
}

void KPNCalc::butHelpClicked(){
	 kapp->invokeHTMLHelp("kpncalc/index.html","");
}

int KPNCalc::validateOperation(int stackHtReq){
	KElement * newElement;
	if (calcStack->stackHeight > stackHtReq-1) {
		if (strlen(lblBuffer->text())!=0) {
				newElement = createNewElement(lblBuffer->text());
				calcStack->push(newElement);
				lblBuffer->setText(NULL);			
		}
		return 1;
	} else {
		if (calcStack->stackHeight == stackHtReq-1 && lblBuffer->text() != NULL) {
			if (strlen(lblBuffer->text())!=0) {
				newElement = createNewElement(lblBuffer->text());
				calcStack->push(newElement);
				lblBuffer->setText(NULL);			
			}	else {
				return 0;
			}
			return 1;
		} else {
			return 0;
		}
	}
}

void KPNCalc::butBaseClicked(){
	if (IOBase==2)
		IOBase = 0;
	else
		IOBase++;

	switch(IOBase){
		case 0: enableBinInput();butBase->setText(i18n("Binary"));break;
		case 1: enableDecInput();butBase->setText(i18n("Decimal"));break;
		case 2: enableHexInput();butBase->setText(i18n("Hexidecimal"));break;
	}
	ReloadGrid();
}
void KPNCalc::butAngleClicked(){
	if (angleUnit==2)
		angleUnit = 0;
	else
		angleUnit++;
	switch(angleUnit){
		case 0: butAngle->setText(i18n("Degrees")); break;
		case 1: butAngle->setText(i18n("Radians"));break;
		case 2: butAngle->setText(i18n("Gradients"));break;
	}
}

void KPNCalc::RefreshStatus(){
	lblStatus->setText(NULL);
}
void KPNCalc::addCharToBuffer(const char * c){
	char * lblText;
	if (lblBuffer->text() != NULL) {
		lblText = new char[1 + strlen(lblBuffer->text())];
		lblText = strcpy(lblText, lblBuffer->text());
		strcat(lblText,c);
	} else {
		lblText = new char[2];
		lblText = strcpy(lblText,c);
	}
	lblBuffer->setText(lblText);
	delete(lblText);
}
void KPNCalc::restoreUndoStack(){
	delete(calcStack);
	undoStack->flip(undoStack->stackHeight-1);
	calcStack = undoStack;
	undoStack = new KStack(*calcStack);
	butUndo->setEnabled(FALSE);
	ReloadGrid();
}
void KPNCalc::saveUndo(){
	if (calcStack->stackHeight > 0) {
		delete(undoStack);
		undoStack = new KStack(*calcStack);
		butUndo->setEnabled(TRUE);
	}
}

int KPNCalc::bufferVerified(){
	if (lblBuffer->text()!=NULL) {
		if (strchr(lblBuffer->text(),'(')) {
			if (strchr(lblBuffer->text(),')')) {
				return 1;
			} else {
				lblStatus->setText("Complex numbers require a close parenthesis.");
				return 0;
			}
		}
		return 1;
	} else {
		lblStatus->setText("Input buffer is empty.");
		return 0;
	}
}

KElement * KPNCalc::createNewElement(const char * sStringRep){
	KElement * newElement;
	if (isdigit(sStringRep[0]) > 0 || (isdigit(sStringRep[1]) > 0 && strchr("-ABCDEF",sStringRep[0])))	{   						
		KScalar * newScalar;
		newScalar = new KScalar(sStringRep, IOBase);
		newElement = new KElement(*newScalar);
	}	else if (strchr(sStringRep,'(')) {
    KComplex * newComplex;
		newComplex = new KComplex(sStringRep, IOBase);
		newElement = new KElement(*newComplex);
	}/*	else if (strchr(sStringRep, '[')) {
		KMatrix * newMatrix;
		newMatrix = new KMatrix(sStringRep, IOBase);
		newElement = new KElement(*newMatrix);
	}*/
	return newElement;
}



